# -*- coding: utf-8 -*-

"""Provider of data related to date and time."""

from calendar import monthrange, timegm
from datetime import date, datetime, time, timedelta
from typing import Any, Final, List, Optional, Union

from mimesis.compat import pytz
from mimesis.data import GMT_OFFSETS, ROMAN_NUMS, TIMEZONES
from mimesis.providers.base import BaseDataProvider
from mimesis.typing import Date, DateTime, Time

__all__ = ["Datetime"]


class Datetime(BaseDataProvider):
    """Class for generating data related to the date and time."""

    # See: https://git.io/Jf15A
    CURRENT_YEAR = datetime.now().year

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes."""
        super().__init__(*args, **kwargs)
        self._datafile = "datetime.json"
        self._load_datafile(self._datafile)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "datetime"

    @staticmethod
    def bulk_create_datetimes(
        date_start: DateTime,
        date_end: DateTime,
        **kwargs: Any,
    ) -> List[DateTime]:
        """Bulk create datetime objects.

        This method creates list of datetime objects from
        ``date_start`` to ``date_end``.

        You can use the following keyword arguments:

        * ``days``
        * ``hours``
        * ``minutes``
        * ``seconds``
        * ``microseconds``

        See :py:class:`datetime.timedelta` for more details.

        :param date_start: Begin of the range.
        :param date_end: End of the range.
        :param kwargs: Keyword arguments for :py:class:`datetime.timedelta`
        :return: List of datetime objects
        :raises: ValueError: When ``date_start``/``date_end`` not passed and
            when ``date_start`` larger than ``date_end``.
        """
        dt_objects = []

        if not date_start and not date_end:
            raise ValueError("You must pass date_start and date_end")

        if date_end < date_start:
            raise ValueError("date_start can not be larger than date_end")

        while date_start <= date_end:
            date_start += timedelta(**kwargs)
            dt_objects.append(date_start)

        return dt_objects

    def week_date(self, start: int = 2017, end: int = CURRENT_YEAR) -> str:
        """Get week number with year.

        :param start: From start.
        :param end: To end.
        :return: Week number.
        """
        year = self.year(start, end)
        week = self.random.randint(1, 52)
        return "{year}-W{week}".format(
            year=year,
            week=week,
        )

    def day_of_week(self, abbr: bool = False) -> str:
        """Get a random day of week.

        :param abbr: Abbreviated day name.
        :return: Day of the week.
        """
        key = "abbr" if abbr else "name"
        days: List[str] = self.extract(["day", key])
        return self.random.choice(days)

    def month(self, abbr: bool = False) -> str:
        """Get a random month.

        :param abbr: Abbreviated month name.
        :return: Month name.
        """
        key = "abbr" if abbr else "name"
        months: List[str] = self.extract(["month", key])
        return self.random.choice(months)

    def year(self, minimum: int = 1990, maximum: int = CURRENT_YEAR) -> int:
        """Generate a random year.

        :param minimum: Minimum value.
        :param maximum: Maximum value.
        :return: Year.
        """
        return self.random.randint(minimum, maximum)

    def century(self) -> str:
        """Get a random century.

        :return: Century.
        """
        return self.random.choice(ROMAN_NUMS)

    def periodicity(self) -> str:
        """Get a random periodicity string.

        :return: Periodicity.
        """
        periodicity: List[str] = self.extract(["periodicity"])
        return self.random.choice(periodicity)

    def date(self, start: int = 2000, end: int = CURRENT_YEAR) -> Date:
        """Generate random date object.

        :param start: Minimum value of year.
        :param end: Maximum value of year.
        :return: Formatted date.
        """
        year = self.random.randint(start, end)
        month = self.random.randint(1, 12)
        day = self.random.randint(1, monthrange(year, month)[1])
        date_object = date(year, month, day)
        return date_object

    def formatted_date(self, fmt: str = "", **kwargs: Any) -> str:
        """Generate random date as string.

        :param fmt: The format of date, if None then use standard
            accepted in the current locale.
        :param kwargs: Keyword arguments for :meth:`~Datetime.date()`
        :return: Formatted date.
        """
        date_obj = self.date(**kwargs)

        if not fmt:
            fmt = self.extract(["formats", "date"])

        return date_obj.strftime(fmt)

    def time(self) -> Time:
        """Generate a random time object.

        :return: ``datetime.time`` object.
        """
        random_time = time(
            self.random.randint(0, 23),
            self.random.randint(0, 59),
            self.random.randint(0, 59),
            self.random.randint(0, 999999),
        )
        return random_time

    def formatted_time(self, fmt: str = "") -> str:
        """Generate string formatted time.

        :param fmt: The format of time, if None then use standard
            accepted in the current locale.
        :return: String formatted time.
        """
        time_obj = self.time()

        if not fmt:
            fmt = self.extract(["formats", "time"])
        return time_obj.strftime(fmt)

    def day_of_month(self) -> int:
        """Generate a random day of month, from 1 to 31.

        :return: Random value from 1 to 31.
        """
        return self.random.randint(1, 31)

    def timezone(self) -> str:
        """Get a random timezone.

        :return: Timezone.
        """
        return self.random.choice(TIMEZONES)

    def gmt_offset(self) -> str:
        """Get a random GMT offset value.

        :return: GMT Offset.
        """
        return self.random.choice(GMT_OFFSETS)

    def datetime(
        self, start: int = 2000, end: int = CURRENT_YEAR, timezone: Optional[str] = None
    ) -> DateTime:
        """Generate random datetime.

        :param start: Minimum value of year.
        :param end: Maximum value of year.
        :param timezone: Set custom timezone (pytz required).
        :return: Datetime
        """
        datetime_obj = datetime.combine(
            date=self.date(start, end),
            time=self.time(),
        )
        if timezone:
            if not pytz:
                raise ImportError("Timezones are supported only with pytz")
            tz = pytz.timezone(timezone)
            datetime_obj = tz.localize(datetime_obj)

        return datetime_obj

    def formatted_datetime(self, fmt: str = "", **kwargs: Any) -> str:
        """Generate datetime string in human readable format.

        :param fmt: Custom format (default is format for current locale)
        :param kwargs: Keyword arguments for :meth:`~Datetime.datetime()`
        :return: Formatted datetime string.
        """
        dt_obj = self.datetime(**kwargs)

        if not fmt:
            date_fmt = self.extract(["formats", "date"])
            time_fmt = self.extract(["formats", "time"])
            fmt = "{} {}".format(date_fmt, time_fmt)

        return dt_obj.strftime(fmt)

    def timestamp(self, posix: bool = True, **kwargs: Any) -> Union[str, int]:
        """Generate random timestamp.

        :param posix: POSIX time.
        :param kwargs: Kwargs for :meth:`~Datetime.datetime()`.
        :return: Timestamp.
        """
        stamp = self.datetime(**kwargs)

        if posix:
            return timegm(stamp.utctimetuple())

        return stamp.strftime("%Y-%m-%dT%H:%M:%SZ")



# -*- coding: utf-8 -*-

"""Contains all the available data providers."""

from mimesis.providers.address import Address
from mimesis.providers.base import BaseDataProvider, BaseProvider
from mimesis.providers.binaryfile import BinaryFile
from mimesis.providers.choice import Choice
from mimesis.providers.code import Code
from mimesis.providers.cryptographic import Cryptographic
from mimesis.providers.date import Datetime
from mimesis.providers.development import Development
from mimesis.providers.file import File
from mimesis.providers.finance import Finance
from mimesis.providers.food import Food
from mimesis.providers.generic import Generic
from mimesis.providers.hardware import Hardware
from mimesis.providers.internet import Internet
from mimesis.providers.numeric import Numeric
from mimesis.providers.path import Path
from mimesis.providers.payment import Payment
from mimesis.providers.person import Person
from mimesis.providers.science import Science
from mimesis.providers.text import Text
from mimesis.providers.transport import Transport

__all__ = [
    # The main class:
    "BaseProvider",
    # Data base class:
    "BaseDataProvider",
    # Data providers:
    "Address",
    "BinaryFile",
    "Finance",
    "Choice",
    "Code",
    "Choice",
    "Datetime",
    "Development",
    "File",
    "Food",
    "Hardware",
    "Internet",
    "Numeric",
    "Path",
    "Payment",
    "Person",
    "Science",
    "Text",
    "Transport",
    "Cryptographic",
    "Generic",
]



# -*- coding: utf-8 -*-

"""Provides data related to numbers."""

from collections import defaultdict
from decimal import Decimal
from typing import Any, DefaultDict, Final, List, Optional

from mimesis.enums import NumType
from mimesis.providers.base import BaseProvider
from mimesis.typing import Matrix

__all__ = ["Numeric"]


class Numeric(BaseProvider):
    """A provider for generating numeric data."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.__increment_dict: DefaultDict[str, int] = defaultdict(int)
        self.__default_accumulator_value: Final = "default"

    class Meta:
        """Class for metadata."""

        name: Final = "numeric"

    def increment(self, accumulator: Optional[str] = None) -> int:
        """Generate incremental number.

        Each call of this method returns incremented number.

        Example:
            >>> self.increment()
            1
            >>> self.increment()
            2
            >>> self.increment(accumulator="abc")
            1
            >>> self.increment()
            3
            >>> self.increment(accumulator="abc")
            2

        :param accumulator: Accumulator name.
        :return: Integer.
        """
        if not accumulator:
            accumulator = self.__default_accumulator_value

        self.__increment_dict[accumulator] += 1
        return self.__increment_dict[accumulator]

    def float_number(
        self, start: float = -1000.0, end: float = 1000.0, precision: int = 15
    ) -> float:
        """Generate random float number in range [start, end].

        :param start: Start range.
        :param end:  End range.
        :param precision: Round a number to a given
            precision in decimal digits, default is 15.
        :return: Float.
        """
        return self.random.uniform(start, end, precision)

    def floats(
        self, start: float = 0, end: float = 1, n: int = 10, precision: int = 15
    ) -> List[float]:
        """Generate a list of random float numbers.

        :param start: Start range.
        :param end: End range.
        :param n: Length of the list.
        :param precision: Round a number to a given
            precision in decimal digits, default is 15.
        :return: The list of floating-point numbers.
        """
        return [self.float_number(start, end, precision) for _ in range(n)]

    def integer_number(self, start: int = -1000, end: int = 1000) -> int:
        """Generate random integer from start to end.

        :param start: Start range.
        :param end: End range.
        :return: Integer.
        """
        return self.random.randint(start, end)

    def integers(self, start: int = 0, end: int = 10, n: int = 10) -> List[int]:
        """Generate a list of random integers.

        Integers can be negative or positive numbers.
        .. note: You can use both positive and negative numbers.

        :param start: Start.
        :param end: End.
        :param n: Length of list.
        :return: List of integers.

        :Example:
            [-20, -19, -18, -17]
        """
        return self.random.randints(n, start, end)

    def complex_number(
        self,
        start_real: float = 0.0,
        end_real: float = 1.0,
        start_imag: float = 0.0,
        end_imag: float = 1.0,
        precision_real: int = 15,
        precision_imag: int = 15,
    ) -> complex:
        """Generate a random complex number.

        :param start_real: Start real range.
        :param end_real: End real range.
        :param start_imag: Start imaginary range.
        :param end_imag: End imaginary range.
        :param precision_real:  Round a real part of
            number to a given precision.
        :param precision_imag:  Round the imaginary part of
            number to a given precision.
        :return: Complex numbers.
        """
        real_part = self.random.uniform(start_real, end_real, precision_real)
        imag_part = self.random.uniform(start_imag, end_imag, precision_imag)
        return complex(real_part, imag_part)

    def complexes(
        self,
        start_real: float = 0,
        end_real: float = 1,
        start_imag: float = 0,
        end_imag: float = 1,
        precision_real: int = 15,
        precision_imag: int = 15,
        n: int = 10,
    ) -> List[complex]:
        """Generate a list of random complex numbers.

        :param start_real: Start real range.
        :param end_real: End real range.
        :param start_imag: Start imaginary range.
        :param end_imag: End imaginary range.
        :param precision_real:  Round a real part of
            number to a given precision.
        :param precision_imag:  Round the imaginary part of
            number to a given precision.
        :param n: Length of the list.
        :return: A list of random complex numbers.
        """
        numbers = []
        for _ in range(n):
            numbers.append(
                self.complex_number(
                    start_real=start_real,
                    end_real=end_real,
                    start_imag=start_imag,
                    end_imag=end_imag,
                    precision_real=precision_real,
                    precision_imag=precision_imag,
                ),
            )
        return numbers

    def decimal_number(self, start: float = -1000.0, end: float = 1000.0) -> Decimal:
        """Generate random decimal number.

        :param start:  Start range.
        :param end: End range.
        :return: :py:class:`decimal.Decimal` object.
        """
        return Decimal.from_float(self.float_number(start, end))

    def decimals(
        self, start: float = 0.0, end: float = 1000.0, n: int = 10
    ) -> List[Decimal]:
        """Generate a decimal number as :py:class:`decimal.Decimal` objects.

        :param start: Start range.
        :param end: End range.
        :param n: Length of the list.
        :return: A list of random decimal numbers.
        """
        return [self.decimal_number(start, end) for _ in range(n)]

    def matrix(
        self, m: int = 10, n: int = 10, num_type: NumType = NumType.FLOAT, **kwargs: Any
    ) -> Matrix:
        """Generate m x n matrix with random numbers.

        This method works with variety of types,
        so you can pass method-specific `**kwargs`.

        See code for more details.

        :param m: Number of rows.
        :param n: Number of columns.
        :param num_type: NumType enum object.
        :param kwargs: Other method-specific arguments.
        :return: A matrix of random numbers.
        """
        key = self.validate_enum(num_type, NumType)
        kwargs.update({"n": n})
        method = getattr(self, key)
        return [method(**kwargs) for _ in range(m)]



# -*- coding: utf-8 -*-

"""Provides pseudo-scientific data."""

from typing import Any, Final, Optional, Tuple

from mimesis.data import SI_PREFIXES, SI_PREFIXES_SYM
from mimesis.enums import MeasureUnit, MetricPrefixSign
from mimesis.providers.base import BaseProvider

__all__ = ["Science"]


class Science(BaseProvider):
    """Class for generating pseudo-scientific data."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param locale: Current language.
        :param seed: Seed.
        """
        super().__init__(*args, **kwargs)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "science"

    def rna_sequence(self, length: int = 10) -> str:
        """Generate a random RNA sequence.

        :param length: Length of block.
        :return: RNA sequence.

        :Example:
            AGUGACACAA
        """
        return self.random.generate_string("UCGA", length)

    def dna_sequence(self, length: int = 10) -> str:
        """Generate a random DNA sequence.

        :param length: Length of block.
        :return: DNA sequence.

        :Example:
            GCTTTAGACC
        """
        return self.random.generate_string("TCGA", length)

    def measure_unit(
        self,
        name: Optional[MeasureUnit] = None,
        symbol: bool = False,
    ) -> str:
        """Get unit name from International System of Units.

        :param name: Enum object UnitName.
        :param symbol: Return only symbol
        :return: Unit.
        """
        result: Tuple[str, str] = self.validate_enum(
            item=name,
            enum=MeasureUnit,
        )

        if symbol:
            return result[1]
        return result[0]

    def metric_prefix(
        self, sign: Optional[MetricPrefixSign] = None, symbol: bool = False
    ) -> str:
        """Get a random prefix for the International System of Units.

        :param sign: Sing of prefix (positive/negative).
        :param symbol: Return the symbol of the prefix.
        :return: Metric prefix for SI measure units.
        :raises NonEnumerableError: if sign is not supported.

        :Example:
            mega
        """
        prefixes = SI_PREFIXES_SYM if symbol else SI_PREFIXES

        key = self.validate_enum(item=sign, enum=MetricPrefixSign)
        return self.random.choice(prefixes[key])



# -*- coding: utf-8 -*-

"""Business data provider."""

from typing import Any, Final, List

from mimesis.data import (
    CRYPTOCURRENCY_ISO_CODES,
    CRYPTOCURRENCY_SYMBOLS,
    CURRENCY_ISO_CODES,
    CURRENCY_SYMBOLS,
    STOCK_EXCHANGES,
    STOCK_NAMES,
    STOCK_TICKERS,
)
from mimesis.providers.base import BaseDataProvider

__all__ = ["Finance"]


class Finance(BaseDataProvider):
    """Class for generating finance data."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self._datafile = "finance.json"
        self._load_datafile(self._datafile)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "finance"

    def company(self) -> str:
        """Get a random company name.

        :return: Company name.
        """
        names: List[str] = self.extract(["company", "name"])

        return self.random.choice(names)

    def company_type(self, abbr: bool = False) -> str:
        """Get a random type of business entity.

        :param abbr: Abbreviated company type.
        :return: Types of business entity.
        """
        key = "abbr" if abbr else "title"

        company_types: List[str] = self.extract(["company", "type", key])
        return self.random.choice(company_types)

    def currency_iso_code(self, allow_random: bool = False) -> str:
        """Get code of the currency for current locale.

        :param allow_random: Get a random ISO code.
        :return: Currency code.
        """
        code: str = self.extract(["currency-code"])

        if allow_random:
            return self.random.choice(CURRENCY_ISO_CODES)
        return code

    def cryptocurrency_iso_code(self) -> str:
        """Get symbol of random cryptocurrency.

        :return: Symbol of cryptocurrency.
        """
        return self.random.choice(CRYPTOCURRENCY_ISO_CODES)

    def currency_symbol(self) -> str:
        """Get a currency symbol for current locale.

        :return: Currency symbol.
        """
        return CURRENCY_SYMBOLS[self.locale]

    def cryptocurrency_symbol(self) -> str:
        """Get a cryptocurrency symbol.

        :return: Symbol of cryptocurrency.
        """
        return self.random.choice(CRYPTOCURRENCY_SYMBOLS)

    def price(self, minimum: float = 500, maximum: float = 1500) -> float:
        """Generate random price.

        :param minimum: Minimum value of price.
        :param maximum: Maximum value of price.
        :return: Price.
        """
        return self.random.uniform(
            minimum,
            maximum,
            precision=2,
        )

    def price_in_btc(self, minimum: float = 0, maximum: float = 2) -> float:
        """Generate random price in BTC.

        :param minimum: Minimum value of price.
        :param maximum: Maximum value of price.
        :return: Price in BTC.
        """
        return self.random.uniform(
            minimum,
            maximum,
            precision=7,
        )

    def stock_ticker(self) -> str:
        """Returns random stock ticker.

        :return: Ticker.
        """
        return self.random.choice(STOCK_TICKERS)

    def stock_name(self) -> str:
        """Returns stock name.

        :return: Stock name.
        """
        return self.random.choice(STOCK_NAMES)

    def stock_exchange(self) -> str:
        """Returns stock exchange name.

        :return: Returns exchange name.
        """
        return self.random.choice(STOCK_EXCHANGES)



# -*- coding: utf-8 -*-

"""Provides data related to paths."""

import sys
from pathlib import PurePosixPath, PureWindowsPath
from typing import Any, Final

from mimesis.data import (
    FOLDERS,
    PLATFORMS,
    PROGRAMMING_LANGS,
    PROJECT_NAMES,
    USERNAMES,
)
from mimesis.providers.base import BaseProvider

__all__ = ["Path"]


class Path(BaseProvider):
    """Class that provides methods and property for generate paths."""

    def __init__(
        self,
        platform: str = sys.platform,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        """Initialize attributes.

        Supported platforms: 'linux', 'darwin', 'win32', 'win64'.

        :param platform: Required platform type.
        """
        super().__init__(*args, **kwargs)
        self.platform = platform
        self._pathlib_home = PureWindowsPath() if "win" in platform else PurePosixPath()
        self._pathlib_home /= PLATFORMS[platform]["home"]

    class Meta:
        """Class for metadata."""

        name: Final[str] = "path"

    def root(self) -> str:
        """Generate a root dir path.

        :return: Root dir.

        :Example:
            /
        """
        return str(self._pathlib_home.parent)

    def home(self) -> str:
        """Generate a home path.

        :return: Home path.

        :Example:
            /home
        """
        return str(self._pathlib_home)

    def user(self) -> str:
        """Generate a random user.

        :return: Path to user.

        :Example:
            /home/oretha
        """
        user = self.random.choice(USERNAMES)
        user = user.capitalize() if "win" in self.platform else user.lower()
        return str(self._pathlib_home / user)

    def users_folder(self) -> str:
        """Generate a random path to user's folders.

        :return: Path.

        :Example:
            /home/taneka/Pictures
        """
        user = self.user()
        folder = self.random.choice(FOLDERS)
        return str(self._pathlib_home / user / folder)

    def dev_dir(self) -> str:
        """Generate a random path to development directory.

        :return: Path.

        :Example:
            /home/sherrell/Development/Python
        """
        user = self.user()
        folder = self.random.choice(["Development", "Dev"])
        stack = self.random.choice(PROGRAMMING_LANGS)
        return str(self._pathlib_home / user / folder / stack)

    def project_dir(self) -> str:
        """Generate a random path to project directory.

        :return: Path to project.

        :Example:
            /home/sherika/Development/Falcon/mercenary
        """
        dev_dir = self.dev_dir()
        project = self.random.choice(PROJECT_NAMES)
        return str(self._pathlib_home / dev_dir / project)



# -*- coding: utf-8 -*-

"""Binary data provider."""

from pathlib import Path
from typing import Any, Final, Union

from mimesis.enums import (
    AudioFile,
    CompressedFile,
    DocumentFile,
    ImageFile,
    VideoFile,
)
from mimesis.providers.base import BaseProvider

__all__ = ["BinaryFile"]


class BinaryFile(BaseProvider):
    """Class for generating binary data"""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param locale: Current locale.
        :param seed: Seed.
        """
        super().__init__(*args, **kwargs)
        self._data_dir = Path(__file__).parent.parent.joinpath("data", "bin")
        self._sample_name: Final[str] = "sample"

    class Meta:
        """Class for metadata."""

        name: Final[str] = "binaryfile"

    def _read_file(
        self,
        *,
        file_type: Union[
            AudioFile,
            CompressedFile,
            DocumentFile,
            ImageFile,
            VideoFile,
        ],
    ) -> bytes:
        file_type = self.validate_enum(file_type, file_type.__class__)
        file_path = self._data_dir.joinpath(f"{self._sample_name}.{file_type}")

        with open(file_path, "rb") as file:
            return file.read()

    def video(self, *, file_type: VideoFile = VideoFile.MP4) -> bytes:
        """Generates video file of given format and returns it as bytes.

        .. note:: This method accepts keyword-only arguments.

        :param file_type: File extension.
        :return: File as a sequence of bytes.
        """
        return self._read_file(file_type=file_type)

    def audio(self, *, file_type: AudioFile = AudioFile.MP3) -> bytes:
        """Generates audio file of given format and returns it as bytes.

        .. note:: This method accepts keyword-only arguments.

        :param file_type: File extension.
        :return: File as a sequence of bytes.
        """
        return self._read_file(file_type=file_type)

    def document(self, *, file_type: DocumentFile = DocumentFile.PDF) -> bytes:
        """Generates document of given format and returns it as bytes.

        .. note:: This method accepts keyword-only arguments.

        :param file_type: File extension.
        :return: File as a sequence of bytes.
        """
        return self._read_file(file_type=file_type)

    def image(self, *, file_type: ImageFile = ImageFile.PNG) -> bytes:
        """Generates image of given format and returns it as bytes.

        .. note:: This method accepts keyword-only arguments.

        :param file_type: File extension.
        :return: File as a sequence of bytes.
        """
        return self._read_file(file_type=file_type)

    def compressed(self, *, file_type: CompressedFile = CompressedFile.ZIP) -> bytes:
        """Generates compressed file of given format and returns it as bytes.

        .. note:: This method accepts keyword-only arguments.

        :param file_type: File extension.
        :return: File as a sequence of bytes.
        """
        return self._read_file(file_type=file_type)



# -*- coding: utf-8 -*-

"""Provides personal data."""

import hashlib
import re
from string import ascii_letters, digits, punctuation
from typing import Any, Final, List, Optional, Sequence, Tuple, Union

from mimesis.data import (
    BLOOD_GROUPS,
    CALLING_CODES,
    EMAIL_DOMAINS,
    GENDER_SYMBOLS,
    USERNAMES,
)
from mimesis.enums import Gender, TitleType
from mimesis.exceptions import NonEnumerableError
from mimesis.providers.base import BaseDataProvider
from mimesis.random import get_random_item

__all__ = ["Person"]


class Person(BaseDataProvider):
    """Class for generating personal data."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param locale: Current locale.
        :param seed: Seed.
        """
        super().__init__(*args, **kwargs)
        self._datafile = "person.json"
        self._load_datafile(self._datafile)
        self._store = {
            "age": 0,
        }

    class Meta:
        """Class for metadata."""

        name: Final[str] = "person"

    def age(self, minimum: int = 16, maximum: int = 66) -> int:
        """Get a random integer value.

        :param maximum: Maximum value of age.
        :param minimum: Minimum value of age.
        :return: Random integer.

        :Example:
            23.
        """
        age = self.random.randint(minimum, maximum)
        self._store["age"] = age
        return age

    def work_experience(self, working_start_age: int = 22) -> int:
        """Get a work experience.

        :param working_start_age: Age then person start to work.
        :return: Depend on previous generated age.
        """
        age = self._store["age"]
        if age == 0:
            age = self.age()

        return max(age - working_start_age, 0)

    def name(self, gender: Optional[Gender] = None) -> str:
        """Generate a random name.

        :param gender: Gender's enum object.
        :return: Name.

        :Example:
            John.
        """
        key = self.validate_enum(gender, Gender)
        names: List[str] = self.extract(["names", key])
        return self.random.choice(names)

    def first_name(self, gender: Optional[Gender] = None) -> str:
        """Generate a random first name.

        ..note: An alias for self.name().

        :param gender: Gender's enum object.
        :return: First name.
        """
        return self.name(gender)

    def surname(self, gender: Optional[Gender] = None) -> str:
        """Generate a random surname.

        :param gender: Gender's enum object.
        :return: Surname.

        :Example:
            Smith.
        """
        surnames: Sequence[str] = self.extract(["surnames"])

        # Surnames separated by gender.
        if isinstance(surnames, dict):
            key = self.validate_enum(gender, Gender)
            surnames = surnames[key]

        return self.random.choice(surnames)

    def last_name(self, gender: Optional[Gender] = None) -> str:
        """Generate a random last name.

        ..note: An alias for self.surname().

        :param gender: Gender's enum object.
        :return: Last name.
        """
        return self.surname(gender)

    def title(
        self, gender: Optional[Gender] = None, title_type: Optional[TitleType] = None
    ) -> str:
        """Generate a random title for name.

        You can generate random prefix or suffix
        for name using this method.

        :param gender: The gender.
        :param title_type: TitleType enum object.
        :return: The title.
        :raises NonEnumerableError: if gender or title_type in incorrect format.

        :Example:
            PhD.
        """
        gender_key = self.validate_enum(gender, Gender)
        title_key = self.validate_enum(title_type, TitleType)

        titles: List[str] = self.extract(["title", gender_key, title_key])
        return self.random.choice(titles)

    def full_name(self, gender: Optional[Gender] = None, reverse: bool = False) -> str:
        """Generate a random full name.

        :param reverse: Return reversed full name.
        :param gender: Gender's enum object.
        :return: Full name.

        :Example:
            Johann Wolfgang.
        """
        if gender is None:
            gender = get_random_item(Gender, rnd=self.random)

        if gender and isinstance(gender, Gender):
            gender = gender
        else:
            raise NonEnumerableError(Gender)

        fmt = "{1} {0}" if reverse else "{0} {1}"
        return fmt.format(
            self.name(gender),
            self.surname(gender),
        )

    def username(
        self, mask: Optional[str] = None, drange: Tuple[int, int] = (1800, 2100)
    ) -> str:
        """Generate username by template.

        You can create many different usernames using masks.

        - **C** stands for capitalized username.
        - **U** stands for uppercase username.
        - **l** stands for lowercase username.
        - **d** stands for digits in username.

        You can also use symbols to separate the different parts
        of the username: **.** **_** **-**

        :param mask: Mask.
        :param drange: Digits range.
        :raises ValueError: If template is not supported.
        :return: Username as string.

        Example:
            >>> username(mask='C_C_d')
            Cotte_Article_1923
            >>> username(mask='U.l.d')
            ELKINS.wolverine.2013
            >>> username(mask='l_l_d', drange=(1900, 2021))
            plasmic_blockader_1907
        """
        if len(drange) != 2:
            raise ValueError("The drange parameter must contain only two integers.")

        if mask is None:
            mask = "l_d"

        required_tags = "CUl"
        tags = re.findall(r"[CUld.\-_]", mask)

        if not any(tag in tags for tag in required_tags):
            raise ValueError(
                "Username mask must contain at least one of these: (C, U, l)."
            )

        final_username = ""
        for tag in tags:
            username = self.random.choice(USERNAMES)
            if tag == "C":
                final_username += username.capitalize()
            if tag == "U":
                final_username += username.upper()
            elif tag == "l":
                final_username += username.lower()
            elif tag == "d":
                final_username += str(self.random.randint(*drange))
            elif tag in "-_.":
                final_username += tag

        return final_username

    def password(self, length: int = 8, hashed: bool = False) -> str:
        """Generate a password or hash of password.

        :param length: Length of password.
        :param hashed: MD5 hash.
        :return: Password or hash of password.

        :Example:
            k6dv2odff9#4h
        """
        text = ascii_letters + digits + punctuation
        password = "".join([self.random.choice(text) for _ in range(length)])

        if hashed:
            md5 = hashlib.md5()
            md5.update(password.encode())
            return md5.hexdigest()

        return password

    def email(
        self,
        domains: Optional[Sequence[str]] = None,
        unique: bool = False,
    ) -> str:
        """Generate a random email.

        :param domains: List of custom domains for emails.
        :param unique: Makes email addresses unique.
        :return: Email address.
        :raises ValueError: if «unique» is True and the provider was seeded.

        :Example:
            foretime10@live.com
        """
        if unique and self.seed is not None:
            raise ValueError(
                "You cannot use «unique» parameter with the seeded provider"
            )

        if not domains:
            domains = EMAIL_DOMAINS

        domain = self.random.choice(domains)

        if not domain.startswith("@"):
            domain = "@" + domain

        if unique:
            name = self.random.randstr(unique)
        else:
            name = self.username(mask="ld")

        return "{name}{domain}".format(
            name=name,
            domain=domain,
        )

    def gender(self, iso5218: bool = False, symbol: bool = False) -> Union[str, int]:
        """Get a random gender.

        Get a random title of gender, code for the representation
        of human sexes is an international standard that defines a
        representation of human sexes through a language-neutral single-digit
        code or symbol of gender.

        :param iso5218:
            Codes for the representation of human sexes is an international
            standard (0 - not known, 1 - male, 2 - female, 9 - not applicable).
        :param symbol: Symbol of gender.
        :return: Title of gender.

        :Example:
            Male
        """
        if iso5218:
            return self.random.choice([0, 1, 2, 9])

        if symbol:
            return self.random.choice(GENDER_SYMBOLS)

        genders: List[str] = self.extract(["gender"])
        return self.random.choice(genders)

    def sex(self, *args: Any, **kwargs: Any) -> Union[str, int]:
        """An alias for method self.gender().

        See docstrings of method self.gender() for details.

        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return: Sex
        """
        return self.gender(*args, **kwargs)

    def height(self, minimum: float = 1.5, maximum: float = 2.0) -> str:
        """Generate a random height in meters.

        :param minimum: Minimum value.
        :param float maximum: Maximum value.
        :return: Height.

        :Example:
            1.85.
        """
        h = self.random.uniform(minimum, maximum)
        return "{:0.2f}".format(h)

    def weight(self, minimum: int = 38, maximum: int = 90) -> int:
        """Generate a random weight in Kg.

        :param minimum: min value
        :param maximum: max value
        :return: Weight.

        :Example:
            48.
        """
        return self.random.randint(minimum, maximum)

    def blood_type(self) -> str:
        """Get a random blood type.

        :return: Blood type (blood group).

        :Example:
            A+
        """
        return self.random.choice(BLOOD_GROUPS)

    def occupation(self) -> str:
        """Get a random job.

        :return: The name of job.

        :Example:
            Programmer.
        """
        jobs: List[str] = self.extract(["occupation"])
        return self.random.choice(jobs)

    def political_views(self) -> str:
        """Get a random political views.

        :return: Political views.

        :Example:
            Liberal.
        """
        views: List[str] = self.extract(["political_views"])
        return self.random.choice(views)

    def worldview(self) -> str:
        """Get a random worldview.

        :return: Worldview.

        :Example:
            Pantheism.
        """
        views: List[str] = self.extract(["worldview"])
        return self.random.choice(views)

    def views_on(self) -> str:
        """Get a random views on.

        :return: Views on.

        :Example:
            Negative.
        """
        views: List[str] = self.extract(["views_on"])
        return self.random.choice(views)

    def nationality(self, gender: Optional[Gender] = None) -> str:
        """Get a random nationality.

        :param gender: Gender.
        :return: Nationality.

        :Example:
            Russian
        """
        nationalities: List[str] = self.extract(["nationality"])

        # Separated by gender
        if isinstance(nationalities, dict):
            key = self.validate_enum(gender, Gender)
            nationalities = nationalities[key]

        return self.random.choice(nationalities)

    def university(self) -> str:
        """Get a random university.

        :return: University name.

        :Example:
            MIT.
        """
        universities: List[str] = self.extract(["university"])
        return self.random.choice(universities)

    def academic_degree(self) -> str:
        """Get a random academic degree.

        :return: Degree.

        :Example:
            Bachelor.
        """
        degrees: List[str] = self.extract(["academic_degree"])
        return self.random.choice(degrees)

    def language(self) -> str:
        """Get a random language.

        :return: Random language.

        :Example:
            Irish.
        """
        languages: List[str] = self.extract(["language"])
        return self.random.choice(languages)

    def telephone(self, mask: str = "", placeholder: str = "#") -> str:
        """Generate a random phone number.

        :param mask: Mask for formatting number.
        :param placeholder: A placeholder for a mask (default is #).
        :return: Phone number.

        :Example:
            +7-(963)-409-11-22.
        """
        if not mask:
            code = self.random.choice(CALLING_CODES)
            default = "{}-(###)-###-####".format(code)
            masks = self.extract(["telephone_fmt"], default=[default])
            mask = self.random.choice(masks)

        return self.random.custom_code(mask=mask, digit=placeholder)

    def identifier(self, mask: str = "##-##/##") -> str:
        """Generate a random identifier by mask.

        With this method you can generate any identifiers that
        you need. Simply select the mask that you need.

        :param mask:
            The mask. Here ``@`` is a placeholder for characters and ``#`` is
            placeholder for digits.
        :return: An identifier.

        :Example:
            07-97/04
        """
        return self.random.custom_code(mask=mask)



# -*- coding: utf-8 -*-

"""Specific data provider for Russia (ru)."""

from datetime import datetime
from typing import Final, List, Optional

from mimesis.builtins.base import BaseSpecProvider
from mimesis.enums import Gender
from mimesis.locales import Locale
from mimesis.typing import Seed

__all__ = ["RussiaSpecProvider"]


class RussiaSpecProvider(BaseSpecProvider):
    """Class that provides special data for Russia (ru)."""

    def __init__(self, seed: Optional[Seed] = None) -> None:
        """Initialize attributes."""
        super().__init__(locale=Locale.RU, seed=seed)
        self._load_datafile(self._datafile)
        self._current_year = str(datetime.now().year)

    class Meta:
        """The name of the provider."""

        name: Final[str] = "russia_provider"

    def generate_sentence(self) -> str:
        """Generate sentence from the parts.

        :return: Sentence.
        """
        sentences = self.extract(["sentence"])
        sentence = [
            self.random.choice(sentences[k]) for k in ("head", "p1", "p2", "tail")
        ]
        return "{0} {1} {2} {3}".format(*sentence)

    def patronymic(self, gender: Optional[Gender] = None) -> str:
        """Generate random patronymic name.

        :param gender: Gender of person.
        :return: Patronymic name.

        :Example:
            Алексеевна.
        """
        gender = self.validate_enum(gender, Gender)
        patronymics: List[str] = self.extract(["patronymic", str(gender)])
        return self.random.choice(patronymics)

    def passport_series(self, year: Optional[int] = None) -> str:
        """Generate random series of passport.

        :param year: Year of manufacture.
        :type year: int or None
        :return: Series.

        :Example:
            02 15.
        """
        if not year:
            year = self.random.randint(10, int(self._current_year[2:]))

        region = self.random.randint(1, 99)
        return "{:02d} {}".format(region, year)

    def passport_number(self) -> int:
        """Generate random passport number.

        :return: Number.

        :Example:
            560430
        """
        return self.random.randint(100000, 999999)

    def series_and_number(self) -> str:
        """Generate a random passport number and series.

        :return: Series and number.

        :Example:
            57 16 805199.
        """
        return "{} {}".format(
            self.passport_series(),
            self.passport_number(),
        )

    def snils(self) -> str:
        """Generate snils with special algorithm.

        :return: SNILS.

        :Example:
            41917492600.
        """
        numbers = []
        control_codes = []

        for i in range(0, 9):
            numbers.append(self.random.randint(0, 9))

        for i in range(9, 0, -1):
            control_codes.append(numbers[9 - i] * i)

        control_code = sum(control_codes)
        code = "".join(str(number) for number in numbers)

        if control_code in (100, 101):
            snils = code + "00"
            return snils

        if control_code < 100:
            snils = code + str(control_code)
            return snils

        if control_code > 101:
            control_code = control_code % 101
            if control_code == 100:
                control_code = 0
            snils = code + "{:02}".format(control_code)
            return snils

    def inn(self) -> str:
        """Generate random, but valid ``INN``.

        :return: INN.
        """

        def control_sum(nums: List[int], t: str) -> int:
            digits_dict = {
                "n2": [7, 2, 4, 10, 3, 5, 9, 4, 6, 8],
                "n1": [3, 7, 2, 4, 10, 3, 5, 9, 4, 6, 8],
            }
            number = 0
            digits = digits_dict[t]

            for i, _ in enumerate(digits, start=0):
                number += nums[i] * digits[i]
            return number % 11 % 10

        numbers = []
        for x in range(0, 10):
            numbers.append(self.random.randint(1 if x == 0 else 0, 9))

        n2 = control_sum(numbers, "n2")
        numbers.append(n2)
        n1 = control_sum(numbers, "n1")
        numbers.append(n1)
        return "".join([str(x) for x in numbers])

    def ogrn(self) -> str:
        """Generate random valid ``OGRN``.

        :return: OGRN.

        :Example:
            4715113303725.
        """
        numbers = []
        for _ in range(0, 12):
            numbers.append(self.random.randint(1 if _ == 0 else 0, 9))

        ogrn = "".join([str(x) for x in numbers])
        check_sum = str(int(ogrn) % 11 % 10)

        return "{}{}".format(ogrn, check_sum)

    def bic(self) -> str:
        """Generate random ``BIC`` (Bank ID Code).

        :return: BIC.

        :Example:
            044025575.
        """
        country_code = "04"
        code = "{:02}".format(self.random.randint(1, 10))
        bank_number = "{:02}".format(self.random.randint(0, 99))
        bank_office = "{:03}".format(self.random.randint(50, 999))
        bic = country_code + code + bank_number + bank_office
        return bic

    def kpp(self) -> str:
        """Generate random ``KPP``.

        :return: 'KPP'.

        :Example:
            560058652.
        """
        tax_codes = [
            "7700",
            "7800",
            "5000",
            "0100",
            "0200",
            "0300",
            "0500",
            "0600",
            "0700",
            "0800",
            "0900",
            "1000",
            "1100",
            "1200",
            "1300",
            "1400",
            "1500",
            "1600",
            "1700",
            "1800",
            "1900",
            "2000",
            "2100",
            "2200",
            "2300",
            "2400",
            "2500",
            "2600",
            "2700",
            "2800",
            "2900",
            "3000",
            "3100",
            "3200",
            "3300",
            "3400",
            "3500",
            "3600",
            "3700",
            "3800",
            "3900",
            "4000",
            "4100",
            "4900",
            "5100",
            "5200",
            "5300",
            "5400",
            "5500",
            "5600",
            "5700",
            "5800",
            "5900",
            "6000",
            "6100",
            "6200",
            "6300",
            "6400",
            "6500",
            "6600",
            "6700",
            "6800",
            "6900",
            "7000",
            "7100",
            "7200",
            "7300",
            "7400",
            "7500",
            "7600",
            "7900",
            "8600",
            "8700",
            "8900",
            "9100",
            "9200",
            "9800",
            "9900",
            "9901",
            "9951",
            "9952",
            "9953",
            "9954",
            "9955",
            "9956",
            "9957",
            "9958",
            "9959",
            "9961",
            "9962",
            "9965",
            "9966",
            "9971",
            "9972",
            "9973",
            "9974",
            "9975",
            "9976",
            "9977",
            "9979",
            "9998",
        ]

        tax_code = tax_codes[self.random.randint(0, len(tax_codes) - 1)]
        reg_code = "{:02}".format(self.random.randint(1, 99))
        reg_number = "{:03}".format(self.random.randint(1, 999))
        kpp = tax_code + reg_code + reg_number
        return kpp



# -*- coding: utf-8 -*-

"""Implements various helpers which are used in the various data providers.

This module contains custom ``Random()`` class where implemented a lot of
methods which are not included in standard :py:class:`random.Random`,
but frequently used in this project.
"""

import os
import random as random_module
import string
import uuid
from typing import Any, List, Optional

__all__ = ["Random", "get_random_item", "random"]


class Random(random_module.Random):
    """Custom class for the possibility of extending.

    The class is a subclass of the class :py:class:`random.Random`
    from the module random of the standard library, which provides the custom methods.

    """

    def randints(self, amount: int = 3, a: int = 1, b: int = 100) -> List[int]:
        """Generate list of random integers.

        :param amount: Amount of elements.
        :param a: Minimum value of range.
        :param b: Maximum value of range.
        :return: List of random integers.
        :raises ValueError: if amount less or equal to zero.
        """
        if amount <= 0:
            raise ValueError("Amount out of range.")

        return [int(self.random() * (b - a)) + a for _ in range(amount)]

    @staticmethod
    def urandom(size: int = 8) -> bytes:
        """Return a bytes object containing random bytes.

        :param size: The size of byte object.
        :return: Bytes.
        """
        return os.urandom(size)

    def generate_string(self, str_seq: str, length: int = 10) -> str:
        """Generate random string created from string sequence.

        :param str_seq: String sequence of letters or digits.
        :param length: Max value.
        :return: Single string.
        """
        return "".join(self.choice(str_seq) for _ in range(length))

    def custom_code(self, mask: str = "@###", char: str = "@", digit: str = "#") -> str:
        """Generate custom code using ascii uppercase and random integers.

        :param mask: Mask of code.
        :param char: Placeholder for characters.
        :param digit: Placeholder for digits.
        :return: Custom code.
        """
        char_code = ord(char)
        digit_code = ord(digit)

        if char_code == digit_code:
            raise ValueError(
                "You cannot use the same placeholder for digits and chars!"
            )

        def random_int(a: int, b: int) -> int:
            b = b - a
            return int(self.random() * b) + a

        _mask = mask.encode()
        code = bytearray(len(_mask))
        for i, p in enumerate(_mask):
            if p == char_code:
                a = random_int(65, 91)  # A-Z
            elif p == digit_code:
                a = random_int(48, 58)  # 0-9
            else:
                a = p
            code[i] = a
        return code.decode()

    def uniform(self, a: float, b: float, precision: int = 15) -> float:
        """Get a random number in the range [a, b) or [a, b] depending on rounding.

        :param a: Minimum value.
        :param b: Maximum value.
        :param precision: Round a number to a given
            precision in decimal digits, default is 15.
        """
        return round(a + (b - a) * self.random(), precision)

    def randstr(self, unique: bool = False, length: Optional[int] = None) -> str:
        """Generate random string value.

        This method can be especially useful when you need to generate
        only unique values in your provider. Just pass parameter unique=True.

        Basically, this method is just a simple wrapper around :py:class:`uuid.UUID`.

        :param unique: Generate only unique values.
        :param length: Length of string. Default range is [a, b].
        :return: Random string.

        """
        if unique:
            return str(uuid.uuid4().hex)

        if length is None:
            length = self.randint(16, 128)

        _string = string.ascii_letters + string.digits
        _string = "".join(self.choice(_string) for _ in range(length))
        return _string


def get_random_item(enum: Any, rnd: Optional[Random] = None) -> Any:
    """Get random item of enum object.

    :param enum: Enum object.
    :param rnd: Custom random object.
    :return: Random item of enum.
    """
    if rnd and isinstance(rnd, Random):
        return rnd.choice(list(enum))
    return random_module.choice(list(enum))


# Compat
# See: https://github.com/lk-geimfari/mimesis/issues/469
random = Random()



# -*- coding: utf-8 -*-

"""Provides data related to payment."""

import re
import string
from typing import Any, Dict, Final, Optional

from mimesis.data import CREDIT_CARD_NETWORKS
from mimesis.enums import CardType, Gender
from mimesis.exceptions import NonEnumerableError
from mimesis.locales import Locale
from mimesis.providers.base import BaseProvider
from mimesis.providers.person import Person
from mimesis.random import get_random_item
from mimesis.shortcuts import luhn_checksum

__all__ = ["Payment"]


class Payment(BaseProvider):
    """Class that provides data related to payments."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param args: Arguments.
        :param kwargs: Keyword arguments.
        """
        super().__init__(*args, **kwargs)
        self._person = Person(Locale.EN, seed=self.seed)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "payment"

    def cid(self) -> str:
        """Generate a random CID.

        :return: CID code.

        :Example:
            7452
        """
        return "{:04d}".format(self.random.randint(1, 9999))

    def paypal(self) -> str:
        """Generate a random PayPal account.

        :return: Email of PapPal user.

        :Example:
            wolf235@gmail.com
        """
        return self._person.email()

    def bitcoin_address(self) -> str:
        """Generate a random bitcoin address.

        :return: Bitcoin address.

        :Example:
            3EktnHQD7RiAE6uzMj2ZifT9YgRrkSgzQX
        """
        type_ = self.random.choice(["1", "3"])
        letters = string.ascii_letters + string.digits
        return type_ + "".join(self.random.choice(letters) for _ in range(33))

    def ethereum_address(self) -> str:
        """Generate a random Ethereum address.

        .. Note: The address will look like Ethereum address,
        but keep in mind that it is not the valid address.

        :return: Ethereum address.

        :Example:
            0xe8ece9e6ff7dba52d4c07d37418036a89af9698d
        """
        bits = self.random.getrandbits(160)
        address = bits.to_bytes(20, byteorder="big")
        return "0x" + address.hex()

    def credit_card_network(self) -> str:
        """Generate a random credit card network.

        :return: Credit card network

        :Example:
            MasterCard
        """
        return self.random.choice(CREDIT_CARD_NETWORKS)

    def credit_card_number(self, card_type: Optional[CardType] = None) -> str:
        """Generate a random credit card number.

        :param card_type: Issuing Network. Default is Visa.
        :return: Credit card number.
        :raises NotImplementedError: if card_type not supported.

        :Example:
            4455 5299 1152 2450
        """
        length = 16
        regex = re.compile(r"(\d{4})(\d{4})(\d{4})(\d{4})")

        if card_type is None:
            card_type = get_random_item(CardType, rnd=self.random)

        if card_type == CardType.VISA:
            number = self.random.randint(4000, 4999)
        elif card_type == CardType.MASTER_CARD:
            number = self.random.choice(
                [
                    self.random.randint(2221, 2720),
                    self.random.randint(5100, 5599),
                ]
            )
        elif card_type == CardType.AMERICAN_EXPRESS:
            number = self.random.choice([34, 37])
            length = 15
            regex = re.compile(r"(\d{4})(\d{6})(\d{5})")
        else:
            raise NonEnumerableError(CardType)

        str_num = str(number)
        while len(str_num) < length - 1:
            str_num += self.random.choice(string.digits)

        groups = regex.search(  # type: ignore
            str_num + luhn_checksum(str_num),
        ).groups()
        card = " ".join(groups)
        return card

    def credit_card_expiration_date(self, minimum: int = 16, maximum: int = 25) -> str:
        """Generate a random expiration date for credit card.

        :param minimum: Date of issue.
        :param maximum: Maximum of expiration_date.
        :return: Expiration date of credit card.

        :Example:
            03/19.
        """
        month = self.random.randint(1, 12)
        year = self.random.randint(minimum, maximum)
        return "{0:02d}/{1}".format(month, year)

    def cvv(self) -> str:
        """Generate a random CVV.

        :return: CVV code.

        :Example:
            069
        """
        return "{:03d}".format(self.random.randint(1, 999))

    def credit_card_owner(
        self,
        gender: Optional[Gender] = None,
    ) -> Dict[str, str]:
        """Generate credit card owner.

        :param gender: Gender of credit card owner.
        :type gender: Gender's enum object.
        :return:
        """
        owner = {
            "credit_card": self.credit_card_number(),
            "expiration_date": self.credit_card_expiration_date(),
            "owner": self._person.full_name(gender=gender).upper(),
        }
        return owner



# -*- coding: utf-8 -*-

"""Specific data provider for Poland (pl)."""

from typing import Final, Optional

from mimesis.builtins.base import BaseSpecProvider
from mimesis.enums import Gender
from mimesis.locales import Locale
from mimesis.providers import Datetime
from mimesis.typing import DateTime, Seed

__all__ = ["PolandSpecProvider"]


class PolandSpecProvider(BaseSpecProvider):
    """Class that provides special data for Poland (pl)."""

    def __init__(self, seed: Optional[Seed] = None) -> None:
        """Initialize attributes."""
        super().__init__(locale=Locale.PL, seed=seed)

    class Meta:
        """The name of the provider."""

        name: Final[str] = "poland_provider"

    def nip(self) -> str:
        """Generate random valid 10-digit NIP.

        :return: Valid 10-digit NIP
        """
        nip_digits = [int(d) for d in str(self.random.randint(101, 998))]
        nip_digits += [self.random.randint(0, 9) for _ in range(6)]
        nip_coefficients = (6, 5, 7, 2, 3, 4, 5, 6, 7)
        sum_v = sum([nc * nd for nc, nd in zip(nip_coefficients, nip_digits)])

        checksum_digit = sum_v % 11
        if checksum_digit > 9:
            return self.nip()
        nip_digits.append(checksum_digit)
        return "".join(str(d) for d in nip_digits)

    def pesel(
        self, birth_date: Optional[DateTime] = None, gender: Optional[Gender] = None
    ) -> str:
        """Generate random 11-digit PESEL.

        :param birth_date: Initial birth date (optional)
        :param gender: Gender of person
        :return: Valid 11-digit PESEL
        """
        date_object = birth_date
        if not date_object:
            date_object = Datetime().datetime(1940, 2018)

        year = date_object.date().year
        month = date_object.date().month
        day = date_object.date().day
        pesel_digits = [int(d) for d in str(year)][-2:]

        if 1800 <= year <= 1899:
            month += 80
        elif 2000 <= year <= 2099:
            month += 20
        elif 2100 <= year <= 2199:
            month += 40
        elif 2200 <= year <= 2299:
            month += 60

        pesel_digits += [int(d) for d in "{:02d}".format(month)]
        pesel_digits += [int(d) for d in "{:02d}".format(day)]
        series_number = self.random.randint(0, 999)
        pesel_digits += [int(d) for d in "{:03d}".format(series_number)]

        if gender == Gender.MALE:
            gender_digit = self.random.choice((1, 3, 5, 7, 9))
        elif gender == Gender.FEMALE:
            gender_digit = self.random.choice((0, 2, 4, 6, 8))
        else:
            gender_digit = self.random.choice(range(10))

        pesel_digits.append(gender_digit)
        pesel_coeffs = (9, 7, 3, 1, 9, 7, 3, 1, 9, 7)
        sum_v = sum([nc * nd for nc, nd in zip(pesel_coeffs, pesel_digits)])
        checksum_digit = sum_v % 10
        pesel_digits.append(checksum_digit)
        return "".join(str(d) for d in pesel_digits)

    def regon(self) -> str:
        """Generate random valid 9-digit REGON.

        :return: Valid 9-digit REGON
        """
        regon_coeffs = (8, 9, 2, 3, 4, 5, 6, 7)
        regon_digits = [self.random.randint(0, 9) for _ in range(8)]
        sum_v = sum([nc * nd for nc, nd in zip(regon_coeffs, regon_digits)])
        checksum_digit = sum_v % 11
        if checksum_digit > 9:
            checksum_digit = 0
        regon_digits.append(checksum_digit)
        return "".join(str(d) for d in regon_digits)



# -*- coding: utf-8 -*-

"""The data provider of a variety of codes."""

from typing import Any, Final, Optional

from mimesis.data import (
    EAN_MASKS,
    IMEI_TACS,
    ISBN_GROUPS,
    ISBN_MASKS,
    LOCALE_CODES,
)
from mimesis.enums import EANFormat, ISBNFormat
from mimesis.locales import Locale
from mimesis.providers.base import BaseProvider
from mimesis.shortcuts import luhn_checksum

__all__ = ["Code"]


class Code(BaseProvider):
    """A class, which provides methods for generating codes."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "code"

    def locale_code(self) -> str:
        """Get a random locale code (MS-LCID).

        See Windows Language Code Identifier Reference
        for more information.

        :return: Locale code.
        """
        return self.random.choice(LOCALE_CODES)

    def issn(self, mask: str = "####-####") -> str:
        """Generate a random ISSN.

        :param mask: Mask of ISSN.
        :return: ISSN.
        """
        return self.random.custom_code(mask=mask)

    def isbn(
        self, fmt: Optional[ISBNFormat] = None, locale: Locale = Locale.DEFAULT
    ) -> str:
        """Generate ISBN for current locale.

        To change ISBN format, pass parameter ``code`` with needed value of
        the enum object :class:`~mimesis.enums.ISBNFormat`

        :param fmt: ISBN format.
        :param locale: Locale code.
        :return: ISBN.
        :raises NonEnumerableError: if code is not enum ISBNFormat.
        """
        fmt_value = self.validate_enum(item=fmt, enum=ISBNFormat)
        mask = ISBN_MASKS[fmt_value].format(ISBN_GROUPS[locale.value])
        return self.random.custom_code(mask)

    def ean(self, fmt: Optional[EANFormat] = None) -> str:
        """Generate EAN.

        To change EAN format, pass parameter ``code`` with needed value of
        the enum object :class:`~mimesis.enums.EANFormat`.

        :param fmt: Format of EAN.
        :return: EAN.
        :raises NonEnumerableError: if code is not enum EANFormat.
        """
        key = self.validate_enum(
            item=fmt,
            enum=EANFormat,
        )
        mask = EAN_MASKS[key]
        return self.random.custom_code(mask=mask)

    def imei(self) -> str:
        """Generate a random IMEI.

        :return: IMEI.
        """
        num = self.random.choice(IMEI_TACS)
        num = num + str(self.random.randint(100000, 999999))
        return num + luhn_checksum(num)

    def pin(self, mask: str = "####") -> str:
        """Generate a random PIN code.

        :param mask: Mask of pin code.
        :return: PIN code.
        """
        return self.random.custom_code(mask=mask)



# -*- coding: utf-8 -*-

"""Provides data related to internet."""

import urllib.error
import urllib.parse
import urllib.request
from ipaddress import IPv4Address, IPv6Address
from typing import Any, Dict, Final, List, Optional, Union

from mimesis.data import (
    EMOJI,
    HTTP_METHODS,
    HTTP_STATUS_CODES,
    HTTP_STATUS_MSGS,
    TLD,
    USER_AGENTS,
    USERNAMES,
)
from mimesis.enums import MimeType, PortRange, TLDType, URLScheme
from mimesis.locales import Locale
from mimesis.providers.base import BaseProvider
from mimesis.providers.date import Datetime
from mimesis.providers.file import File
from mimesis.providers.text import Text
from mimesis.typing import Keywords

__all__ = ["Internet"]


class Internet(BaseProvider):
    """Class for generating data related to the internet."""

    _MAX_IPV4: Final[int] = (2 ** 32) - 1
    _MAX_IPV6: Final[int] = (2 ** 128) - 1

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize attributes.

        :param args: Arguments.
        :param kwargs: Keyword arguments.
        """
        super().__init__(*args, **kwargs)
        self.file = File(seed=self.seed)
        self.text = Text(locale=Locale.EN, seed=self.seed)
        self.datetime = Datetime(locale=Locale.EN)

    class Meta:
        """Class for metadata."""

        name: Final[str] = "internet"

    def content_type(self, mime_type: Optional[MimeType] = None) -> str:
        """Get a random HTTP content type.

        :return: Content type.

        :Example:
            Content-Type: application/json
        """
        fmt = self.file.mime_type(type_=mime_type)
        return "Content-Type: {}".format(fmt)

    def http_status_message(self) -> str:
        """Get a random HTTP status message.

        :return: HTTP status message.

        :Example:
            200 OK
        """
        return self.random.choice(HTTP_STATUS_MSGS)

    def http_status_code(self) -> int:
        """Get a random HTTP status code.

        :return: HTTP status.

        :Example:
            200
        """
        return self.random.choice(HTTP_STATUS_CODES)

    def http_method(self) -> str:
        """Get a random HTTP method.

        :return: HTTP method.

        :Example:
            POST
        """
        return self.random.choice(HTTP_METHODS)

    def ip_v4_object(self) -> IPv4Address:
        """Generate random :py:class:`ipaddress.IPv4Address` object.

        :return: :py:class:`ipaddress.IPv4Address` object.
        """
        return IPv4Address(
            self.random.randint(0, self._MAX_IPV4),
        )

    def ip_v4_with_port(self, port_range: PortRange = PortRange.ALL) -> str:
        """Generate a random IPv4 address as string.

        :param port_range: PortRange enum object.
        :return: IPv4 address as string.

        :Example:
            19.121.223.58:8000
        """
        return "{}:{}".format(self.ip_v4(), self.port(port_range))

    def ip_v4(self) -> str:
        """Generate a random IPv4 address as string.

        :Example:
            19.121.223.58
        """
        return str(self.ip_v4_object())

    def ip_v6_object(self) -> IPv6Address:
        """Generate random :py:class:`ipaddress.IPv6Address` object.

        :return: :py:class:`ipaddress.IPv6Address` object.
        """
        return IPv6Address(
            self.random.randint(
                0,
                self._MAX_IPV6,
            ),
        )

    def ip_v6(self) -> str:
        """Generate a random IPv6 address as string.

        :return: IPv6 address string.

        :Example:
            2001:c244:cf9d:1fb1:c56d:f52c:8a04:94f3
        """
        return str(self.ip_v6_object())

    def mac_address(self) -> str:
        """Generate a random MAC address.

        :return: Random MAC address.

        :Example:
            00:16:3e:25:e7:f1
        """
        mac_hex = [
            0x00,
            0x16,
            0x3E,
            self.random.randint(0x00, 0x7F),
            self.random.randint(0x00, 0xFF),
            self.random.randint(0x00, 0xFF),
        ]
        mac = ["{:02x}".format(x) for x in mac_hex]
        return ":".join(mac)

    def emoji(self) -> str:
        """Get a random emoji shortcut code.

        :return: Emoji code.

        :Example:
            :kissing:
        """
        return self.random.choice(EMOJI)

    @staticmethod
    def image_placeholder(
        width: Union[int, str] = 1920, height: Union[int, str] = 1080
    ) -> str:
        """Generate a link to the image placeholder.

        :param width: Width of image.
        :param height: Height of image.
        :return: URL to image placeholder.
        """
        url = "http://placehold.it/{width}x{height}"
        return url.format(width=width, height=height)

    @staticmethod
    def stock_image(
        width: Union[int, str] = 1920,
        height: Union[int, str] = 1080,
        keywords: Optional[Keywords] = None,
        writable: bool = False,
    ) -> Union[str, bytes]:
        """Generate random stock image (JPG/JPEG) hosted on Unsplash.

        See «Random search term» on https://source.unsplash.com/
        for more details.

        .. note:: This method required an active HTTP connection
            if you want to get a writable object.

        :param width: Width of the image.
        :param height: Height of the image.
        :param keywords: List of search keywords.
        :param writable: Return image as sequence ob bytes.
        :return: Link to the image.
        """
        api_url = "https://source.unsplash.com/{}x{}?{}"

        if keywords is not None:
            keywords_str = ",".join(keywords)
        else:
            keywords_str = ""

        url = api_url.format(width, height, keywords_str)

        if writable:
            try:
                response = urllib.request.urlopen(url)
                content: bytes = response.read()
                return content
            except urllib.error.URLError:
                raise urllib.error.URLError("Required an active HTTP connection")
        return url

    def hashtags(self, quantity: int = 4) -> List[str]:
        """Generate a list of hashtags.

        :param quantity: The quantity of hashtags.
        :return: The list of hashtags.
        :raises NonEnumerableError: if category is not in Hashtag.

        :Example:
            ['#love', '#sky', '#nice']
        """

        if quantity < 1:
            raise ValueError("Quantity must be a positive integer.")

        return ["#" + self.text.word() for _ in range(quantity)]

    def hostname(
        self,
        tld_type: Optional[TLDType] = None,
        subdomains: Optional[List[str]] = None,
    ) -> str:
        """Generate a random hostname without scheme.

        :param tld_type: TLDType.
        :param subdomains: List of subdomains (make sure they are valid).
        :return: Hostname.
        """
        tld = self.tld(tld_type=tld_type)
        host = self.random.choice(USERNAMES)

        if subdomains:
            subdomain = self.random.choice(subdomains)
            host = f"{subdomain}.{host}"

        return f"{host}{tld}"

    def url(
        self,
        scheme: Optional[URLScheme] = URLScheme.HTTPS,
        port_range: Optional[PortRange] = None,
        tld_type: Optional[TLDType] = None,
        subdomains: Optional[List[str]] = None,
    ) -> str:
        """Generate random URL.

        :param scheme: Scheme.
        :param port_range: PortRange enum object.
        :param tld_type: TLDType.
        :param subdomains: List of subdomains (make sure they are valid).
        :return: URL.
        """
        host = self.hostname(tld_type, subdomains)
        url_scheme = self.validate_enum(scheme, URLScheme)

        url = f"{url_scheme}://{host}"

        if port_range is not None:
            url = f"{url}:{self.port(port_range)}"

        return f"{url}/"

    def uri(
        self,
        scheme: Optional[URLScheme] = URLScheme.HTTPS,
        port_range: Optional[PortRange] = None,
        tld_type: Optional[TLDType] = None,
        subdomains: Optional[List[str]] = None,
        query_params_count: Optional[int] = None,
    ) -> str:
        """Generate a random URI.

        :param scheme: Scheme.
        :param port_range: PortRange enum object.
        :param tld_type: TLDType.
        :param subdomains: List of subdomains (make sure they are valid).
        :param query_params_count: Query params.
        :return: URI.
        """
        directory = (
            self.datetime.date(start=2010, end=self.datetime.CURRENT_YEAR)
            .strftime("%Y-%m-%d")
            .replace("-", "/")
        )
        url = self.url(scheme, port_range, tld_type, subdomains)
        uri = f"{url}{directory}/{self.slug()}"

        if query_params_count:
            uri += self.query_string(query_params_count)

        return uri

    def query_string(self, length: Optional[int] = None) -> str:
        """Generate arbitrary query string of given length.

        :param length: Length of query string.
        :return: Query string.
        """
        return urllib.parse.urlencode(self.query_parameters(length))

    def query_parameters(self, length: Optional[int] = None) -> Dict[str, str]:
        """Generate arbitrary query parameters as a dict.

        :param length: Length of dictionary (key/value pair).
        :return: Dict of query parameters.
        """

        if not length:
            length = self.random.randint(1, 10)

        return dict(zip(self.text.words(length), self.text.words(length)))

    def top_level_domain(self, tld_type: Optional[TLDType] = None) -> str:
        """Generates random top level domain.

        :param tld_type: Enum object :class:`enums.TLDType`
        :return: Top level domain.
        :raises NonEnumerableError: if tld_type not in :class:`enums.TLDType`.
        """
        key = self.validate_enum(item=tld_type, enum=TLDType)
        return self.random.choice(TLD[key])

    def tld(self, *args: Any, **kwargs: Any) -> str:
        """Generates random top level domain.

        An alias for :meth:`top_level_domain`
        """
        return self.top_level_domain(*args, **kwargs)

    def user_agent(self) -> str:
        """Get a random user agent.

        :return: User agent.

        :Example:
            Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0)
            Gecko/20100101 Firefox/15.0.1
        """
        return self.random.choice(USER_AGENTS)

    def port(self, port_range: PortRange = PortRange.ALL) -> int:
        """Generate random port.

        :param port_range: PortRange enum object.
        :return: Port number.
        :raises NonEnumerableError: if port_range is not in PortRange.

        :Example:
            8080
        """

        rng = self.validate_enum(port_range, PortRange)
        return self.random.randint(*rng)

    def slug(self, parts_count: Optional[int] = None) -> str:
        """Generate a random slug of given parts count.

        :param parts_count: Slug's parts count.
        :return: Slug.
        """

        if not parts_count:
            parts_count = self.random.randint(2, 12)

        if parts_count > 12:
            raise ValueError("Slug's parts count must be <= 12")

        if parts_count < 2:
            raise ValueError("Slug must contain more than 2 parts")

        return "-".join(self.text.words(parts_count))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mimesis',
 'mimesis.builtins',
 'mimesis.data',
 'mimesis.data.int',
 'mimesis.providers']

package_data = \
{'': ['*'],
 'mimesis.data': ['bin/*',
                  'cs/*',
                  'da/*',
                  'de-at/*',
                  'de-ch/*',
                  'de/*',
                  'el/*',
                  'en-au/*',
                  'en-ca/*',
                  'en-gb/*',
                  'en/*',
                  'es-mx/*',
                  'es/*',
                  'et/*',
                  'fa/*',
                  'fi/*',
                  'fr/*',
                  'hu/*',
                  'is/*',
                  'it/*',
                  'ja/*',
                  'kk/*',
                  'ko/*',
                  'nl-be/*',
                  'nl/*',
                  'no/*',
                  'pl/*',
                  'pt-br/*',
                  'pt/*',
                  'ru/*',
                  'sk/*',
                  'sv/*',
                  'tr/*',
                  'uk/*',
                  'zh/*']}

setup_kwargs = {
    'name': 'mimesis',
    'version': '5.0.0',
    'description': 'Mimesis: fake data generator.',
    'long_description': "Mimesis - Fake Data Generator\n-----------------------------\n\n|\n\n.. image:: https://raw.githubusercontent.com/lk-geimfari/mimesis/master/.github/images/octopus-no-retina-sm.png\n     :target: https://github.com/lk-geimfari/mimesis\n\n|\n\nDescription\n-----------\n\n.. image:: https://github.com/lk-geimfari/mimesis/actions/workflows/test.yml/badge.svg?branch=master\n     :target: https://github.com/lk-geimfari/mimesis/actions/workflows/test.yml?query=branch%3Amaster\n     :alt: Github Actions Test\n\n.. image:: https://readthedocs.org/projects/mimesis/badge/?version=latest\n     :target: https://mimesis.name/\n     :alt: Documentation Status\n\n.. image:: https://codecov.io/gh/lk-geimfari/mimesis/branch/master/graph/badge.svg\n     :target: https://codecov.io/gh/lk-geimfari/mimesis\n     :alt: Code Coverage\n\n.. image:: https://www.codefactor.io/repository/github/lk-geimfari/mimesis/badge\n   :target: https://www.codefactor.io/repository/github/lk-geimfari/mimesis\n   :alt: CodeFactor\n\n.. image:: https://img.shields.io/pypi/v/mimesis?color=bright-green\n     :target: https://pypi.org/project/mimesis/\n     :alt: PyPi Version\n\n.. image:: https://img.shields.io/pypi/dm/mimesis\n     :target: https://pypi.org/project/mimesis/\n     :alt: PyPI - Downloads\n\n.. image:: https://img.shields.io/badge/python-3.8%20%7C%203.9-brightgreen\n     :target: https://pypi.org/project/mimesis/\n     :alt: Python version\n\nMimesis (/mɪˈmiːsɪs, Ancient Greek: μίμησις, mīmēsis) is a high-performance fake data generator for Python,\nwhich provides data for a variety of purposes in a variety of languages. The fake data could be used to populate\na testing database, create fake API endpoints, create JSON and XML files of arbitrary structure, anonymize data taken\nfrom production and etc.\n\nInstallation\n------------\n\n\nTo install mimesis, simply use pip:\n\n.. code:: text\n\n    ~ ⟩ pip install mimesis\n\n\nPython compatibility\n---------------------\n\nMimesis 5.0 supports Python 3.8, 3.9, and 3.10.\n\nThe Mimesis 4.1.3 is the last to support Python 3.6 and 3.7.\n\nSupported Features\n------------------\n\n- **Easy**: Designed to be easy to use and learn.\n- **Multilingual**: Supports data for `a lot of languages <https://mimesis.name/getting_started.html#locales>`_.\n- **Performance**: The `fastest <https://mimesis.name/foreword.html#performance>`_ data generator available for Python.\n- **Data variety**: Supports `a lot of data providers <https://mimesis.name/api.html>`_ for a variety of purposes.\n- **Country-specific data providers**: Provides data specific only for `some countries <https://mimesis.name/api.html#builtin-data-providers>`_.\n- **Extensibility**: You can create your own data providers and use them with Mimesis.\n- **Generic data provider**: The `simplified <https://mimesis.name/getting_started.html#generic-provider>`_ access to all the providers from a single object.\n- **Zero hard dependencies**: Does not require any modules other than the Python standard library.\n- **Schema-based generators**: Provides an easy mechanism to generate data by the schema of any complexity.\n\n\nDocumentation\n-------------\n\nYou can find the complete documentation on the `Read the Docs <https://mimesis.name>`_.\n\nIt is divided into several sections:\n\n-  `Foreword <https://mimesis.name/foreword.html>`_\n-  `Getting Started <https://mimesis.name/getting_started.html>`_\n-  `Tips and Tricks <https://mimesis.name/tips.html>`_\n-  `API Reference <https://mimesis.name/api.html>`_\n-  `Contributing <https://mimesis.name/contributing.html>`_\n-  `Changelog <https://mimesis.name/changelog.html>`_\n\nYou can improve it by sending pull requests to this repository.\n\nUsage\n-----\n\nThis readme file contains information for Mimesis 5.0.0 which is not released yet, so If you're \nlooking for actual docs for 4.1.3 then see: https://mimesis.name/\n\nThis library is really easy to use and everything you need is just import an object which\nrepresents a type of data you need (we call such object a *Provider*).\n\nIn the example below we import provider `Person <https://mimesis.name/api.html#person>`_,\nwhich represents data related to personal information, such as name, surname, email and etc:\n\n.. code:: python\n\n    >>> from mimesis import Person\n    >>> from mimesis.locales import Locale\n    >>> person = Person(Locale.EN)\n\n    >>> person.full_name()\n    'Brande Sears'\n\n    >>> person.email(domains=['example.com'])\n    'roccelline1878@example.com'\n\n    >>> person.email(domains=['mimesis.name'], unique=True)\n    'f272a05d39ec46fdac5be4ac7be45f3f@mimesis.name'\n\n    >>> person.telephone(mask='1-4##-8##-5##3')\n    '1-436-896-5213'\n\n\nMore about the other providers you can read in our `documentation`_.\n\n.. _documentation: https://mimesis.name/getting_started.html#providers\n\n\nLocales\n-------\n\nMimesis currently includes support for 34 different `locales`_. You can\nspecify a locale when creating providers and they will return data that\nis appropriate for the language or country associated with that locale.\n\nLet's take a look how it works:\n\n.. code:: python\n\n    >>> from mimesis import Person\n    >>> from mimesis.locales import Locale\n    >>> from mimesis.enums import Gender\n\n    >>> de = Person(locale=Locale.DE)\n    >>> en = Person(locale=Locale.EN)\n\n    >>> de.full_name(gender=Gender.FEMALE)\n    'Sabrina Gutermuth'\n\n    >>> en.full_name(gender=Gender.MALE)\n    'Layne Gallagher'\n\n\n.. _locales: https://mimesis.name/getting_started.html#locales\n\nProviders\n---------\n\nMimesis support over twenty different data providers available,\nwhich can produce data related to people, food, computer hardware,\ntransportation, addresses, internet and more.\n\nSee `Data Providers <https://mimesis.name/getting_started.html#data-providers>`_ for more info.\n\nHow to Contribute\n-----------------\n\n1. Take a look at `contributing guidelines`_.\n2. Check for open issues or open a fresh issue to start a discussion\n   around a feature idea or a bug.\n3. Fork the repository on GitHub to start making your changes to the\n   *your_branch* branch.\n4. Add yourself to the list of `contributors`_.\n5. Send a pull request and bug the maintainer until it gets merged and\n   published.\n\n.. _contributing guidelines: https://github.com/lk-geimfari/mimesis/blob/master/CONTRIBUTING.rst\n.. _contributors: https://github.com/lk-geimfari/mimesis/blob/master/CONTRIBUTORS.rst\n\n\nUseful links\n------------\n\nI have a Telegram channel where I daily post news, announces and all the open-source\ngoodies I found, so subscribe: `@software_dev_channel <https://t.me/software_dev_channel>`_.\n\nDisclaimer\n----------\n\nThe authors of `Mimesis` do not assume any responsibility for how you use it or how you use data generated with it.\nThis library was designed with good intentions to make testing easier. Do not use the data generated with Mimesis for illegal purposes.\n\nLicense\n-------\n\nMimesis is licensed under the MIT License. See `LICENSE`_ for more\ninformation.\n\n.. _LICENSE: https://github.com/lk-geimfari/mimesis/blob/master/LICENSE\n",
    'author': 'Isaak Uchakaev',
    'author_email': 'likid.geimfari@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/lk-geimfari/mimesis',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

"""Provides all the data related to science."""

SI_PREFIXES = {
    "negative": [
        "deci",
        "centi",
        "milli",
        "micro",
        "nano",
        "pico",
        "femto",
        "atto",
        "zepto",
        "yocto",
    ],
    "positive": [
        "yotta",
        "zetta",
        "exa",
        "peta",
        "tera",
        "giga",
        "mega",
        "kilo",
        "hecto",
        "deca",
    ],
}

SI_PREFIXES_SYM = {
    "negative": ["d", "c", "m", "μ", "n", "p", "f", "a", "z", "y"],
    "positive": ["Y", "Z", "E", "P", "T", "G", "M", "k", "h", "da"],
}



# -*- coding: utf-8 -*-

"""Implements classes for generating data by schema."""
import warnings
from typing import Any, Callable, Final, Iterator, List, Optional, Sequence

from mimesis.exceptions import FieldError, SchemaError
from mimesis.locales import Locale
from mimesis.providers.generic import Generic
from mimesis.typing import JSON, SchemaType, Seed

__all__ = ["BaseField", "Field", "Schema"]


class BaseField:
    """
    BaseField is a class for generating data by the name of the method.

    Instance of this object takes any string which represents the name
    of any method of any supported data provider (:class:`~mimesis.Generic`)
    and the ``**kwargs`` of the method.

    See :class:`~mimesis.schema.BaseField.perform` for more details.
    """

    class Meta:
        base = True

    def __init__(
        self,
        locale: Locale = Locale.DEFAULT,
        seed: Optional[Seed] = None,
        providers: Optional[Sequence[Any]] = None,
    ) -> None:
        """Initialize field.

        :param locale: Locale
        :param seed: Seed for random.
        """
        self._gen = Generic(locale, seed)

        if providers:
            self._gen.add_providers(*providers)

        self._table = {}  # type: ignore

    def perform(
        self,
        name: Optional[str] = None,
        key: Optional[Callable[[Any], Any]] = None,
        **kwargs: Any
    ) -> Any:
        """Performs the value of the field by its name.

        It takes any string which represents the name of any method of
        any supported data provider and the ``**kwargs`` of this method.

        .. note:: Some data providers have methods with the same names
            and in such cases, you can explicitly define that the method
            belongs to data-provider ``name='provider.name'`` otherwise
            it will return the data from the first provider which
            has a method ``name``.

        You can apply a *key function* to the result returned by
        the method, bt passing a parameter **key** with a callable
        object which returns the final result.

        :param name: Name of the method.
        :param key: A key function (or any other callable object)
            which will be applied to result.
        :param kwargs: Kwargs of method.
        :return: Value which represented by method.
        :raises ValueError: if provider not
            supported or if field not defined.
        """
        if name is None:
            raise FieldError()

        def tail_parser(tails: str, obj: Any) -> Any:
            """Return method from end of tail.

            :param tails: Tail string
            :param obj: Search tail from this object
            :return last tailed method
            """
            provider_name, method_name = tails.split(".", 1)

            if "." in method_name:
                raise FieldError(name)

            attr = getattr(obj, provider_name)
            if attr is not None:
                try:
                    return getattr(attr, method_name)
                except AttributeError:
                    raise FieldError(name)

        try:
            if name not in self._table:
                if "." not in name:
                    # Fix https://github.com/lk-geimfari/mimesis/issues/619
                    if name == self._gen.choice.Meta.name:
                        self._table[name] = self._gen.choice
                    else:
                        for provider in dir(self._gen):
                            provider = getattr(self._gen, provider)
                            if name in dir(provider):
                                self._table[name] = getattr(provider, name)
                else:
                    self._table[name] = tail_parser(name, self._gen)

            result = self._table[name](**kwargs)
            if key and callable(key):
                return key(result)
            return result
        except KeyError:
            raise FieldError(name)

    def __str__(self) -> str:
        return "{} <{}>".format(self.__class__.__name__, self._gen.locale)


class Field(BaseField):
    """Greedy field.

    The field whcih evaluates immediately.

    Example:
        >>> _ = Field()
        >>> _('username')
        Dogtag_1836
    """

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        return self.perform(*args, **kwargs)


class Schema:
    """Class which return list of filled schemas."""

    _MIN_ITERATIONS_VALUE: Final = 1

    __slots__ = ("_schema",)

    def __init__(self, schema: SchemaType) -> None:
        """Initialize schema.

        :param schema: A schema (must be a callable object).
        """
        if schema and callable(schema):
            self._schema = schema
        else:
            raise SchemaError()

    def create(self, iterations: int = 1) -> List[JSON]:
        """Creates a list of a fulfilled schemas.

        .. note::
            This method evaluates immediately, so be careful on creating
            large datasets otherwise you're risking running out of memory.

            If you need a lazy version of this method, see
            :meth:`iterator`

        :param iterations: Number of iterations.
        :return: List of fulfilled schemas.
        """

        if iterations < self._MIN_ITERATIONS_VALUE:
            raise ValueError("The number of iterations must be greater than 0.")

        return [self._schema() for _ in range(iterations)]

    def loop(self) -> Iterator[JSON]:
        """Fulfills a schema **infinitely** in a lazy way.

        This method can be useful when you have some dynamic
        conditions in depend on which the generation must be interrupted.

        If you accepting all risks below and want to suppress
        the warnings then use :py:class:`warnings.catch_warnings`

        .. note::
            Since data `mimesis` provides are limited, frequent calls of
            this method can cause data duplication.

        .. note::
            Before using this method, ask yourself: **Do I really need this**?

            In most cases, the answer is: Nah, :meth:`iterator` is enough.

        .. warning::
            Do not use this method without interrupt conditions, otherwise,
            you're risking running out of memory.

        .. warning::
            **Never** (seriously) call `list()`, `tuple()` or any other callable which tries to
            evaluate the whole lazy object on this method — infinite called infinite
            for a reason.

        :return: An infinite iterator with fulfilled schemas.
        """

        warnings.warn(
            "You're iterating over the infinite object! "
            "The schema.loop() can cause a serious memory leak."
            "Please, see: https://mimesis.name/api.html#mimesis.schema.Schema.loop"
        )

        while True:
            yield self._schema()

    def iterator(self, iterations: int = 1) -> Iterator[JSON]:
        """Fulfills schema in a lazy way.

        :param iterations: Number of iterations.
        :return: List of fulfilled schemas.
        """

        if iterations < self._MIN_ITERATIONS_VALUE:
            raise ValueError("The number of iterations must be greater than 0.")

        for item in range(iterations):
            yield self._schema()



# -*- coding: utf-8 -*-

"""Provides all the data related to the development."""

LICENSES = [
    "Apache License, 2.0 (Apache-2.0)",
    "The BSD 3-Clause License",
    "The BSD 2-Clause License",
    "GNU General Public License (GPL)",
    "General Public License (LGPL)",
    "MIT License (MIT)",
    "Mozilla Public License 2.0 (MPL-2.0)",
    "Common Development and Distribution License (CDDL-1.0)",
    "Eclipse Public License (EPL-1.0)",
]

PROGRAMMING_LANGS = [
    "ASP",
    "Assembly",
    "AutoIt",
    "Awk",
    "Bash",
    "C",
    "C Shell",
    "C#",
    "C++",
    "Caml",
    "Ceylon",
    "Clojure",
    "CoffeeScript",
    "Common Lisp",
    "D",
    "Dart",
    "Delphi",
    "Dylan",
    "ECMAScript",
    "Elixir",
    "Emacs Lisp",
    "Erlang",
    "F#",
    "Falcon",
    "Fortran",
    "GNU Octave",
    "Go",
    "Groovy",
    "Haskell",
    "haXe",
    "Io",
    "J#",
    "Java",
    "JavaScript",
    "Julia",
    "Kotlin",
    "Lisp",
    "Lua",
    "Mathematica",
    "Objective-C",
    "OCaml",
    "Perl",
    "PHP",
    "PL-I",
    "PL-SQL",
    "PowerShell",
    "Prolog",
    "Python",
    "R",
    "Racket",
    "Ruby",
    "Rust",
    "Scala",
    "Scheme",
    "Smalltalk",
    "Tcl",
    "Tex",
    "Transact-SQL",
    "TypeScript",
    "Z shell",
]

OS = [
    "Arch",
    "CentOS",
    "Debian",
    "Fedora",
    "FreeBSD",
    "Gentoo",
    "Kali",
    "Lubuntu",
    "Manjaro",
    "Mint",
    "OS X",
    "macOS",
    "OpenBSD",
    "PCLinuxOS",
    "Slackware",
    "Ubuntu",
    "Windows 10",
    "Windows 7",
    "Windows 8",
    "Windows 8.1",
    "Zorin",
    "elementaryOS",
    "macOS",
    "openSUSE",
]

FOLDERS = [
    "Development",
    "Downloads",
    "Documents",
    "Music",
    "Video",
    "Work",
    "Pictures",
    "Desktop",
    "Study",
]

PROJECT_NAMES = [
    "aardonyx",
    "abelisaurus",
    "achelousaurus",
    "achillobator",
    "acrocanthosaurus",
    "aegyptosaurus",
    "afrovenator",
    "agilisaurus",
    "alamosaurus",
    "albertaceratops",
    "albertosaurus",
    "alectrosaurus",
    "alioramus",
    "allosaurus",
    "alvarezsaurus",
    "amargasaurus",
    "ammosaurus",
    "ampelosaurus",
    "amygdalodon",
    "anatotitan",
    "anchiceratops",
    "anchisaurus",
    "ankylosaurus",
    "anserimimus",
    "antarctopelta",
    "antarctosaurus",
    "apatosaurus",
    "aragosaurus",
    "aralosaurus",
    "archaeoceratops",
    "archaeopteryx",
    "archaeornithomimus",
    "argentinosaurus",
    "arrhinoceratops",
    "atlascopcosaurus",
    "aucasaurus",
    "austrosaurus",
    "avaceratops",
    "avalonia",
    "avimimus",
    "azendohsaurus",
    "bactrosaurus",
    "bagaceratops",
    "bambiraptor",
    "barapasaurus",
    "barosaurus",
    "baryonyx",
    "becklespinax",
    "beipiaosaurus",
    "bellusaurus",
    "borogovia",
    "brachiosaurus",
    "brachyceratops",
    "bugenasaura",
    "buitreraptor",
    "camarasaurus",
    "camptosaurus",
    "carnotaurus",
    "caudipteryx",
    "cedarpelta",
    "centrosaurus",
    "ceratosaurus",
    "cetiosauriscus",
    "cetiosaurus",
    "chaoyangsaurus",
    "chasmosaurus",
    "chialingosaurus",
    "chindesaurus",
    "chinshakiangosaurus",
    "chirostenotes",
    "chubutisaurus",
    "chungkingosaurus",
    "citipati",
    "coelophysis",
    "coelurus",
    "coloradisaurus",
    "compsognathus",
    "conchoraptor",
    "confuciusornis",
    "corythosaurus",
    "cryolophosaurus",
    "dacentrurus",
    "daspletosaurus",
    "datousaurus",
    "deinocheirus",
    "deinonychus",
    "deltadromeus",
    "diceratops",
    "dicraeosaurus",
    "dilophosaurus",
    "diplodocus",
    "dracorex",
    "dravidosaurus",
    "dromaeosaurus",
    "dromiceiomimus",
    "dryosaurus",
    "dryptosaurus",
    "dubreuillosaurus",
    "edmontonia",
    "edmontosaurus",
    "einiosaurus",
    "elaphrosaurus",
    "emausaurus",
    "eolambia",
    "eoraptor",
    "eotyrannus",
    "equijubus",
    "erketu",
    "erlikosaurus",
    "euhelopus",
    "euoplocephalus",
    "europasaurus",
    "euskelosaurus",
    "eustreptospondylus",
    "fukuiraptor",
    "fukuisaurus",
    "gallimimus",
    "gargoyleosaurus",
    "garudimimus",
    "gasosaurus",
    "gasparinisaura",
    "gastonia",
    "giganotosaurus",
    "gilmoreosaurus",
    "giraffatitan",
    "gobisaurus",
    "gorgosaurus",
    "goyocephale",
    "graciliceratops",
    "gryposaurus",
    "guaibasaurus",
    "guanlong",
    "hadrosaurus",
    "hagryphus",
    "haplocanthosaurus",
    "harpymimus",
    "herrerasaurus",
    "hesperosaurus",
    "heterodontosaurus",
    "homalocephale",
    "huayangosaurus",
    "hylaeosaurus",
    "hypacrosaurus",
    "hypselosaurus",
    "hypsilophodon",
    "iguanodon",
    "indosuchus",
    "ingenia",
    "irritator",
    "isisaurus",
    "janenschia",
    "jaxartosaurus",
    "jingshanosaurus",
    "jinzhousaurus",
    "jobaria",
    "juravenator",
    "kentrosaurus",
    "khaan",
    "kotasaurus",
    "kritosaurus",
    "lamaceratops",
    "lambeosaurus",
    "lapparentosaurus",
    "leaellynasaura",
    "leptoceratops",
    "lesothosaurus",
    "lexovisaurus",
    "liaoceratops",
    "liaoxiornis",
    "ligabuesaurus",
    "liliensternus",
    "lophorhothon",
    "lophostropheus",
    "lufengosaurus",
    "lurdusaurus",
    "lycorhinus",
    "magyarosaurus",
    "maiasaura",
    "majungatholus",
    "malawisaurus",
    "mamenchisaurus",
    "mapusaurus",
    "marshosaurus",
    "masiakasaurus",
    "massospondylus",
    "maxakalisaurus",
    "megalosaurus",
    "melanorosaurus",
    "metriacanthosaurus",
    "microceratops",
    "micropachycephalosaurus",
    "microraptor",
    "minmi",
    "monolophosaurus",
    "mononykus",
    "mussaurus",
    "muttaburrasaurus",
    "nanotyrannus",
    "nanshiungosaurus",
    "nemegtosaurus",
    "neovenator",
    "neuquenosaurus",
    "nigersaurus",
    "nipponosaurus",
    "noasaurus",
    "nodosaurus",
    "nomingia",
    "nothronychus",
    "nqwebasaurus",
    "omeisaurus",
    "ornitholestes",
    "ornithomimus",
    "orodromeus",
    "oryctodromeus",
    "othnielia",
    "ouranosaurus",
    "oviraptor",
    "rebbachisaurus",
    "rhabdodon",
    "rhoetosaurus",
    "rinchenia",
    "riojasaurus",
    "rugops",
    "saichania",
    "saltasaurus",
    "saltopus",
    "sarcosaurus",
    "saurolophus",
    "sauropelta",
    "saurophaganax",
    "saurornithoides",
    "scelidosaurus",
    "scutellosaurus",
    "secernosaurus",
    "segisaurus",
    "segnosaurus",
    "seismosaurus",
    "shamosaurus",
    "shanag",
    "shantungosaurus",
    "shunosaurus",
    "shuvuuia",
    "silvisaurus",
    "sinocalliopteryx",
    "sinornithosaurus",
    "sinosauropteryx",
    "sinraptor",
    "sinvenator",
    "zalmoxes",
    "zephyrosaurus",
    "zuniceratops",
    "byzantine",
    "svengali",
    "accolade",
    "acrimony",
    "angst",
    "anomaly",
    "antidote",
    "baroque",
    "bona_fide",
    "bourgeois",
    "bravado",
    "brogue",
    "brusque",
    "cacophony",
    "caustic",
    "charisma",
    "cloying",
    "deja-vu",
    "dichotomy",
    "elan",
    "ennui",
    "epitome",
    "esoteric",
    "euphemism",
    "faux pas",
    "fiasco",
    "finagle",
    "glib",
    "harbinger",
    "hedonist",
    "heresy",
    "idyllic",
    "insidious",
    "junket",
    "kitsch",
    "litany",
    "lurid",
    "malaise",
    "malinger",
    "mantra",
    "maudlin",
    "mercenary",
    "misnomer",
    "nirvana",
    "oblivion",
    "ogle",
    "ostracize",
    "panacea",
    "paradox",
    "peevish",
    "propriety",
    "revel",
    "rhetoric",
    "spartan",
    "stigma",
    "stoic",
    "suave",
    "sycophant",
    "tirade",
    "tryst",
    "untenable",
    "vicarious",
    "vile",
    "waft",
    "zealous",
]



# -*- coding: utf-8 -*-

"""Specific data provider for Brazil (pt-br)."""
from typing import Final, List, Optional

from mimesis.builtins.base import BaseSpecProvider
from mimesis.locales import Locale
from mimesis.typing import Seed

__all__ = ["BrazilSpecProvider"]


# TODO: Refactor this piece of s... some code.
class BrazilSpecProvider(BaseSpecProvider):
    """Class that provides special data for Brazil (pt-br)."""

    def __init__(self, seed: Optional[Seed] = None) -> None:
        """Initialize attributes."""
        super().__init__(locale=Locale.PT_BR, seed=seed)

    class Meta:
        """The name of the provider."""

        name: Final[str] = "brazil_provider"

    def cpf(self, with_mask: bool = True) -> str:
        """Get a random CPF.

        :param with_mask: Use CPF mask (###.###.###-##).
        :returns: Random CPF.

        :Example:
            001.137.297-40
        """

        def get_verifying_digit_cpf(cpf: List[int], peso: int) -> int:
            """Calculate the verifying digit for the CPF.

            :param cpf: List of integers with the CPF.
            :param peso: Integer with the weight for the modulo 11 calculate.
            :returns: The verifying digit for the CPF.
            """
            soma = 0
            for index, digit in enumerate(cpf):
                soma += digit * (peso - index)
            resto = soma % 11
            if resto == 0 or resto == 1 or resto >= 11:
                return 0
            return 11 - resto

        cpf_without_dv = [self.random.randint(0, 9) for _ in range(9)]
        first_dv = get_verifying_digit_cpf(cpf_without_dv, 10)

        cpf_without_dv.append(first_dv)
        second_dv = get_verifying_digit_cpf(cpf_without_dv, 11)
        cpf_without_dv.append(second_dv)

        cpf = "".join([str(i) for i in cpf_without_dv])

        if with_mask:
            return cpf[:3] + "." + cpf[3:6] + "." + cpf[6:9] + "-" + cpf[9:]
        return cpf

    def cnpj(self, with_mask: bool = True) -> str:
        """Get a random CNPJ.

        :param with_mask: Use cnpj mask (###.###.###-##)
        :returns: Random cnpj.

        :Example:
            77.732.230/0001-70
        """

        def get_verifying_digit_cnpj(cnpj: List[int], peso: int) -> int:
            """Calculate the verifying digit for the CNPJ.

            :param cnpj: List of integers with the CNPJ.
            :param peso: Integer with the weight for the modulo 11 calculate.
            :returns: The verifying digit for the CNPJ.
            """
            soma = 0
            if peso == 5:
                peso_list = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
            elif peso == 6:
                peso_list = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
            for i, _ in enumerate(cnpj):
                soma += peso_list[i] * cnpj[i]
            resto = soma % 11
            if resto < 2:
                return 0
            return 11 - resto

        cnpj_without_dv = [self.random.randint(0, 9) for _ in range(12)]

        first_dv = get_verifying_digit_cnpj(cnpj_without_dv, 5)
        cnpj_without_dv.append(first_dv)

        second_dv = get_verifying_digit_cnpj(cnpj_without_dv, 6)
        cnpj_without_dv.append(second_dv)

        cnpj = "".join([str(i) for i in cnpj_without_dv])

        if with_mask:
            return "{}.{}.{}/{}-{}".format(
                cnpj[:2], cnpj[2:5], cnpj[5:8], cnpj[8:12], cnpj[12:]
            )
        return cnpj
